#include "ell/StateAcceptor.hh"
#include "biu/RandomNumberFactory.hh" // for SA_Metropolis
#include <cmath> // exp for SA_Metropolis


namespace ell {

	bool
	SA_All::accept(const StateCollector* const sc, const State& succ) const
	{
		return true;
	}
	
	
	bool
	SA_Le::accept(const StateCollector* const sc, const State& succ) const
	{
		return succ.operator<(*(sc->getLastAdded()));
	}
	
	bool
	SA_E_Eq::accept(const StateCollector* const sc, const State& succ) const
	{
		return ( succ.getEnergy() == sc->getLastAdded()->getEnergy());
	}
	
	bool
	SA_E_Gr::accept(const StateCollector* const sc, const State& succ) const
	{
		return ( succ.getEnergy() > sc->getLastAdded()->getEnergy());
	}
	
	bool
	SA_E_GrEq::accept(const StateCollector* const sc, const State& succ) const
	{
		return ( succ.getEnergy() >= sc->getLastAdded()->getEnergy());
	}
	
	bool
	SA_E_Le::accept(const StateCollector* const sc, const State& succ) const
	{
		return ( succ.getEnergy() < sc->getLastAdded()->getEnergy());
	}
	
	bool
	SA_E_LeEq::accept(const StateCollector* const sc, const State& succ) const
	{
		return ( succ.getEnergy() <= sc->getLastAdded()->getEnergy());
	}
	
	SA_Max_E::SA_Max_E( const double MaxEnergy) : MAX_ENERGY(MaxEnergy) {}
	bool
	SA_Max_E::accept(const StateCollector* const sc, const State& succ) const
	{
		return ( succ.getEnergy() <= MAX_ENERGY);
	}
	
	
	SA_Metropolis::SA_Metropolis(const double _kT) : kT(_kT) {}
	
	bool
	SA_Metropolis::accept(const StateCollector* const sc, const State& succ) const
	{	
		// compute energy difference
		double deltaE = succ.getEnergy() - sc->getLastAdded()->getEnergy();
		  // check if negative energy difference --> always accept
		if (deltaE < 0)
			return true;
		// compute probability for accepting state -> Metropolis
		double p_accept = exp( - deltaE / kT);
		
		// compute random number 0 <= r <= 1
		double r = (((double) biu::RNF::getRN()) / ((double) biu::RNF::getMaxRN())); 
	
		// return true with probability p_accept
		if ( r < p_accept ) return true;
		else return false;
	}

} // namespace ell
